/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the graphics subsystem.
*
*   This template is responsible to access the graphics subsystem within an
*   embedded Linux environment. The display is accessed by using EGL via the
*   graphics subsystem (e.g. Wayland, DRM, X11 or fbdev).
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation and declarations according your particular hardware.
*
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <wayland-client.h>
#include <wayland-server.h>
#include <wayland-client-protocol.h>
#include <wayland-egl.h>
#include <GLES2/gl2.h>
#include <EGL/egl.h>

#include "ewrte.h"
#include "ew_bsp_display.h"
#include "ew_bsp_touch.h"
#include "gfx_system.h"
#include "xdg-shell.h"

#define NO_OF_FINGERS                   1

/* additional touch flag to indicate idle state */
#define EW_BSP_TOUCH_IDLE               0

/* additional touch flag to indicate hold state */
#define EW_BSP_TOUCH_HOLD               4


static int ScreenWidth          = 0;   /* screen width  */
static int ScreenHeight         = 0;   /* screen height */
static int ScreenTransformation = 0;   /* screen transformation (rotate and flip) */

/* single-touch events are provided by the Wayland window manager,
   multi-touch seems to be not provided per finger - only gestures */
static int TouchEventXPos;
static int TouchEventYPos;
static int TouchDown = 0;

static XTouchEvent   TouchEvent[ NO_OF_FINGERS ];
static unsigned char TouchState              = EW_BSP_TOUCH_IDLE;


static struct wl_surface*       Surface       = NULL;
static struct xdg_wm_base*      XdgBase       = NULL;
static struct xdg_surface*      XdgSurface    = NULL;
static struct xdg_toplevel*     XdgToplevel   = NULL;
static struct wl_display*       Display       = NULL;
static struct wl_compositor*    Compositor    = NULL;
static struct wl_subcompositor* SubCompositor = NULL;
static struct wl_seat*          Seat          = NULL;
static struct wl_pointer*       Pointer       = NULL;
static struct wl_touch*         Touch         = NULL;
static struct wl_output*        Output        = NULL;


static EGLDisplay               EglDisplay    = 0;
static EGLSurface               EglSurface    = 0;
static EGLContext               EglContext    = 0;
static EGLConfig                EglConfig     = 0;

static GLint                    FrameBuffer   = 0;
static int                      SurfaceWidth  = 0;
static int                      SurfaceHeight = 0;
static int                      GuiWidth      = 0;
static int                      GuiHeight     = 0;


/***************************1***************************************************
 * private functions
 *******************************************************************************/

static void TouchHandleDown( void* data, struct wl_touch* wl_touch, uint32_t serial,
  uint32_t time, struct wl_surface* surface, int32_t id, wl_fixed_t sx, wl_fixed_t sy )
{
  TouchEventXPos = wl_fixed_to_int( sx );
  TouchEventYPos = wl_fixed_to_int( sy );
  TouchDown = 1;
}


static void TouchHandleUp( void* data, struct wl_touch* wl_touch, uint32_t serial,
  uint32_t time, int32_t id )
{
  TouchDown = 0;
}


static void TouchHandleMotion( void* data, struct wl_touch* wl_touch, uint32_t time,
  int32_t id, wl_fixed_t sx, wl_fixed_t sy )
{
  TouchEventXPos = wl_fixed_to_int( sx );
  TouchEventYPos = wl_fixed_to_int( sy );
}


static void TouchHandleFrame( void* data, struct wl_touch* wl_touch )
{
}


static void TouchHandleCancel( void* data, struct wl_touch* wl_touch )
{
}


static const struct wl_touch_listener TouchListener =
{
  TouchHandleDown,
  TouchHandleUp,
  TouchHandleMotion,
  TouchHandleFrame,
  TouchHandleCancel
};


static void PointerHandleEnter( void* data, struct wl_pointer* pointer,
  uint32_t serial, struct wl_surface* surface, wl_fixed_t sx,
  wl_fixed_t sy )
{
}


static void PointerHandleLeave( void* data, struct wl_pointer* pointer,
  uint32_t serial, struct wl_surface* surface )
{
}


static void PointerHandleMotion( void* data, struct wl_pointer* pointer,
  uint32_t time, wl_fixed_t sx, wl_fixed_t sy )
{
  TouchEventXPos = wl_fixed_to_int( sx );
  TouchEventYPos = wl_fixed_to_int( sy );
}


static void PointerHandleButton( void* data, struct wl_pointer* wl_pointer,
  uint32_t serial, uint32_t time, uint32_t button, uint32_t state )
{
  if ( state )
    TouchDown = 1;
  else
    TouchDown = 0;
}


static void PointerHandleAxis( void* data, struct wl_pointer* wl_pointer,
  uint32_t time, uint32_t axis, wl_fixed_t value )
{
}


static const struct wl_pointer_listener PointerListener =
{
  PointerHandleEnter,
  PointerHandleLeave,
  PointerHandleMotion,
  PointerHandleButton,
  PointerHandleAxis
};


static void SeatHandler( void* data, struct wl_seat* seat,
  enum wl_seat_capability caps )
{
  /* get an interface to the pointer device aka mouse */
  if (( caps & WL_SEAT_CAPABILITY_POINTER ) && !Pointer )
  {
    Pointer = wl_seat_get_pointer( seat );
    wl_pointer_add_listener( Pointer, &PointerListener, NULL );
  }
  else if ( !( caps & WL_SEAT_CAPABILITY_POINTER ) && Pointer )
  {
    wl_pointer_destroy( Pointer );
    Pointer = NULL;
  }

  /* get an interface to the touch device */
  if (( caps & WL_SEAT_CAPABILITY_TOUCH ) && ! Touch )
  {
    Touch = wl_seat_get_touch( seat );
    wl_touch_add_listener( Touch, &TouchListener, NULL );
  }
  else if ( !( caps & WL_SEAT_CAPABILITY_TOUCH ) && Touch )
  {
    wl_touch_destroy( Touch );
    Touch = NULL;
  }
}


static const struct wl_seat_listener SeatListener =
{
  SeatHandler
};


static void OutputGeometryHandler( void *data, struct wl_output *wl_output,
  int32_t x, int32_t y, int32_t physical_width, int32_t physical_height,
  int32_t subpixel, const char *make, const char *model, int32_t transform )
{
  /*
  printf( "OutputGeometryHandler - subpixel=%d, width=%d, height=%d, transform=%d\n",
    subpixel, physical_width, physical_height, transform );
  */

  ScreenTransformation = transform;
}


static void OutputModeHandler( void *data, struct wl_output *wl_output,
  uint32_t flags, int32_t width, int32_t height, int32_t refresh )
{
  /*
  printf( "OutputModeHandler - flags=0x%08X, width=%d, height=%d, refresh=%d\n",
    flags, width, height, refresh );
  */

  /* retrieve currently used resolution from last output mode flag current is set */
  if ( flags & WL_OUTPUT_MODE_CURRENT && width > 0 && height > 0 )
  {
      ScreenWidth  = width;
      ScreenHeight = height;
  }
}


static void HandlePing( void *data, struct xdg_wm_base *xdg_wm_base, uint32_t serial )
{
  xdg_wm_base_pong( xdg_wm_base, serial );
}


static const struct xdg_wm_base_listener XdgBaseListener =
{
  HandlePing
};


static const struct wl_output_listener OutputListener =
{
  OutputGeometryHandler,
  OutputModeHandler
};


static void GlobalRegistryHandler( void* data, struct wl_registry* registry,
  uint32_t id, const char*  interface, uint32_t version )
{
  // printf( "Got a registry event for %s id %d\n", interface, id );

  if ( strcmp( interface, wl_compositor_interface.name ) == 0 )
  {
    Compositor = wl_registry_bind( registry, id, &wl_compositor_interface, 1 );
  }
  else if (strcmp(interface, wl_subcompositor_interface.name) == 0)
  {
      SubCompositor = wl_registry_bind(registry, id, &wl_subcompositor_interface, 1);
  }
  else if ( strcmp( interface, "xdg_wm_base" ) == 0 )
  {
    XdgBase = wl_registry_bind( registry, id, &xdg_wm_base_interface, 1 );
    xdg_wm_base_add_listener( XdgBase, &XdgBaseListener, NULL );
  }
  else if ( strcmp( interface, wl_seat_interface.name ) == 0 )
  {
    Seat = wl_registry_bind( registry, id, &wl_seat_interface, 1 );
    wl_seat_add_listener( Seat, &SeatListener, NULL );
  }
  else if ( strcmp( interface, wl_output_interface.name ) == 0 )
  {
    Output = wl_registry_bind( registry, id, &wl_output_interface, 1 );
    wl_output_add_listener( Output, &OutputListener, NULL );
  }
}


static void GlobalRegistryRemover( void* data, struct wl_registry* registry,
  uint32_t id )
{
  // printf( "Got a registry losing event for %d\n", id );
}


static const struct wl_registry_listener RegistryListener =
{
  GlobalRegistryHandler,
  GlobalRegistryRemover
};

static void HandleConfigure( void *data, struct xdg_surface *xdg_surface, uint32_t serial )
{
  xdg_surface_ack_configure( xdg_surface, serial );
}

static const struct xdg_surface_listener XdgSurfaceListener =
{
  HandleConfigure
};


/*******************************************************************************
* FUNCTION:
*   GfxSystemInit
*
* DESCRIPTION:
*   The function GfxSystemInit is responsible to get an access to the
*   graphics subsystem. In case that the underlying graphics subsystem contains
*   a window manager, the given size is used to create a suitable window.
*   Otherwise, the given size is ignored, because the size of the framebuffer
*   cannot be changed.
*
* ARGUMENTS:
*   NONE.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int GfxSystemInit( void )
{
  struct wl_registry* registry = NULL;

  Display = wl_display_connect( NULL );
  if ( Display == NULL )
  {
    printf( "GfxSystemInit: Cannot connect to display!\n" );
    return 0;
  }

  registry = wl_display_get_registry( Display );
  wl_registry_add_listener( registry, &RegistryListener, NULL );

  wl_display_dispatch( Display );
  wl_display_roundtrip( Display );

  if ( Compositor == NULL )
  {
    printf( "GfxSystemInit: standard wayland interface 'wl_compositor' not available, aborting!\n" );
    return 0;
  }

  if ( XdgBase == NULL )
  {
    printf( "GfxSystemInit: standard wayland interface 'xdg_wm_base' not available, aborting!\n" );
    return 0;
  }

  Surface = wl_compositor_create_surface( Compositor );
  if ( Surface == NULL )
  {
    printf( "GfxSystemInit: Cannot create surface!\n" );
    return 0;
  }

  XdgSurface = xdg_wm_base_get_xdg_surface( XdgBase, Surface );
  if ( XdgSurface == NULL )
  {
    printf( "GfxSystemInit: XDG surface unavailable from compositor surface, aborting!\n" );
    return 0;
  }

  XdgToplevel = xdg_surface_get_toplevel( XdgSurface );
  if ( XdgToplevel == NULL )
  {
    printf( "GfxSystemInit: XDG toplevel unavailable from XDG surface, aborting!\n" );
    return 0;
  }

  xdg_surface_add_listener( XdgSurface, &XdgSurfaceListener, NULL );
  xdg_toplevel_set_fullscreen( XdgToplevel, NULL );

  wl_surface_commit( Surface );


  /* egl init */
  EGLint      contextAttribs[]  = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE };
  EGLint      num_configs;
  EGLint      configAttribs[] =
  {
    EGL_SURFACE_TYPE,      EGL_WINDOW_BIT,
    EGL_RENDERABLE_TYPE,   EGL_OPENGL_ES2_BIT,
    EGL_BUFFER_SIZE,       32,
    EGL_COLOR_BUFFER_TYPE, EGL_RGB_BUFFER,
    EGL_RED_SIZE,          8,
    EGL_GREEN_SIZE,        8,
    EGL_BLUE_SIZE,         8,
    EGL_ALPHA_SIZE,        8,
    EGL_NATIVE_RENDERABLE, EGL_TRUE,
    EGL_NONE
  };

  /* initalize EGL sub-system */
  EglDisplay = eglGetDisplay(( EGLNativeDisplayType ) Display );
  eglInitialize( EglDisplay, 0, 0 );
  eglBindAPI( EGL_OPENGL_ES_API );
  eglChooseConfig( EglDisplay, configAttribs, &EglConfig, 1, &num_configs );

  if ( num_configs != 1 )
    return 0;

  EglContext   = eglCreateContext( EglDisplay, EglConfig, EGL_NO_CONTEXT, contextAttribs );

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemDone
*
* DESCRIPTION:
*   The function GfxSystemDone is responsible to close the access to the
*   graphics subsystem.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void GfxSystemDone( void )
{
  if ( Display != NULL )
    wl_display_disconnect( Display );
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemProcess
*
* DESCRIPTION:
*   The function GfxSystemProcess drives the graphics subsystem or window
*   manager in order to process all pending events.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   1, if further processing is needed, 0 otherwise.
*
*******************************************************************************/
int GfxSystemProcess( void )
{
  wl_display_roundtrip( Display );
  return( wl_display_dispatch_pending( Display ) != -1 );
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemGetInfo
*
* DESCRIPTION:
*   The function GfxSystemGetInfo returns the graphic system info struct.
*
* ARGUMENTS:
*   aGfxSystemInfo  - Pointer to return the graphic system info struct.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void GfxSystemGetInfo( XGfxSystemInfo* aGfxSystemInfo )
{
  if ( ! aGfxSystemInfo )
    return;

  memset( aGfxSystemInfo, 0, sizeof( *aGfxSystemInfo ));
  aGfxSystemInfo->Type          = GFX_SYSTEM_WAYLAND;
  aGfxSystemInfo->FrameBuffer   = (void*)(intptr_t)FrameBuffer;
  aGfxSystemInfo->NativeDisplay = Display;
  aGfxSystemInfo->Display       = EglDisplay;
  aGfxSystemInfo->Surface       = EglSurface;
  aGfxSystemInfo->Context       = EglContext;
  aGfxSystemInfo->Width         = SurfaceWidth;
  aGfxSystemInfo->Height        = SurfaceHeight;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemCreateSurface
*
* DESCRIPTION:
*   The function GfxSystemCreateSurface creates a graphic surface with the
*   given dimensions.
*
* ARGUMENTS:
*   aWidth  - Required surface width.
*   aHeight - Required surface height.
*
* RETURN VALUE:
*   Returns 1, if native window could be created, otherwise 0.
*
*******************************************************************************/
int GfxSystemCreateSurface( int aWidth, int aHeight )
{
  void * nativeWindow;

  GuiWidth  = aWidth;
  GuiHeight = aHeight;

  #if EW_VIEWPORT_AUTOSCALE == 1

  if ( ScreenTransformation == WL_OUTPUT_TRANSFORM_90
    || ScreenTransformation == WL_OUTPUT_TRANSFORM_270
    || ScreenTransformation == WL_OUTPUT_TRANSFORM_FLIPPED_90
    || ScreenTransformation == WL_OUTPUT_TRANSFORM_FLIPPED_270 )
  {
    SurfaceWidth  = ScreenHeight;
    SurfaceHeight = ScreenWidth;
  }
  else
  {
    SurfaceWidth  = ScreenWidth;
    SurfaceHeight = ScreenHeight;
  }

  #else

    SurfaceWidth  = aWidth;
    SurfaceHeight = aHeight;

  #endif

  nativeWindow = wl_egl_window_create( Surface, SurfaceWidth, SurfaceHeight );
  EglSurface   = eglCreateWindowSurface( EglDisplay, EglConfig, nativeWindow, 0 );

  #if EW_PERFORM_FULLSCREEN_UPDATE == 0
    EGLBoolean  eglResult;
    eglResult = eglSurfaceAttrib( EglDisplay, EglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_PRESERVED );
    if ( eglResult != EGL_TRUE )
      printf( "\nWARNING: egl swap behaviour - buffer preservation not supported (egl error: 0x%04X)\n" \
        "Use macro EW_PERFORM_FULLSCREEN_UPDATE = 1\n\n", eglGetError());
  #else
    eglSurfaceAttrib( EglDisplay, EglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_DESTROYED );
  #endif

  eglMakeCurrent( EglDisplay, EglSurface, EglSurface, EglContext );

  /* get the framebuffer and its size in pixel */
  glGetIntegerv( GL_FRAMEBUFFER_BINDING, &FrameBuffer);

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemCreateSubSurface
*
* DESCRIPTION:
*   The function GfxSystemCreateSubSurface creates a sub surface that is located
*   below the main surface.
*
* ARGUMENTS:
*   aWidth  - Required surface width.
*   aHeight - Required surface height.
*
* RETURN VALUE:
*   Returns 1, if native window could be created, otherwise 0.
*
*******************************************************************************/

void* GfxSystemCreateSubSurface( int aWidth, int aHeight )
{
  struct wl_surface*      newSurface;
  struct wl_subsurface*   subSurface;
  EGLSurface              newEglSurface = NULL;
  EGLNativeWindowType     newEglWindow  = NULL;


  /* create new surface */
  newSurface = wl_compositor_create_surface( Compositor );
  if ( ! newSurface )
  {
    printf( "GfxSystemCreateSubSurface: Cannot create new surface!\n" );
    return 0;
  }

  subSurface = wl_subcompositor_get_subsurface( SubCompositor, newSurface, Surface );
  if ( ! subSurface )
  {
    printf( "GfxSystemCreateSubSurface: Cannot get sub surface!\n" );
    return 0;
  }

  wl_subsurface_place_below( subSurface, Surface );
  wl_subsurface_set_desync( subSurface );

  // configure and temorary use video surface - needed to have frame buffer(s) assigned
  newEglWindow  = wl_egl_window_create( newSurface, 1, 1 );
  newEglSurface = eglCreateWindowSurface( EglDisplay, EglConfig, newEglWindow, 0 );

  eglMakeCurrent( EglDisplay, newEglSurface, newEglSurface, EglContext );
  eglSwapBuffers( EglDisplay, newEglSurface );

  eglMakeCurrent( EglDisplay, EglSurface, EglSurface, EglContext );

  return ( void * ) newSurface;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemDestroySubSurface
*
* DESCRIPTION:
*   The function GfxSystemDestroySubSurface the given sub surface.
*
* ARGUMENTS:
*   aSubSurface  - SubSurface handle.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/

void GfxSystemDestroySubSurface( void * aSubSurface )
{
  wl_surface_destroy( aSubSurface );
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemUpdateSurface
*
* DESCRIPTION:
*   The function GfxSystemUpdateSurface updates the graphic surface.
*
* ARGUMENTS:
*   aSurfaceContext  - Surface context
*   aX1              - Surface update area left.
*   aY1              - Surface update area topt.
*   aX2              - Surface update area right.
*   aY2              - Surface update area bottom.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void GfxSystemUpdateSurface ( void * aSurfaceContext, int aX1, int aY1, int aX2, int aY2 )
{
  /* perform the swap if there was something drawn on the screen */
  eglSwapBuffers( EglDisplay, EglSurface );
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemGetTouchEvents
*
* DESCRIPTION:
*   The function GfxSystemGetTouchEvents returns the current touch position and
*   touch status of the different fingers. The returned number of touch events
*   indicates the number of XTouchEvent that contain position and status
*   information.
*   The orientation of the touch positions is adjusted to match GUI coordinates.
*   If the hardware supports only single touch, the finger number is always 0.
*
* ARGUMENTS:
*   aToucEvents  - Pointer to return array of XTouchEvent.
*
* RETURN VALUE:
*   Returns the number of available touch events.
*
*******************************************************************************/
int GfxSystemGetTouchEvents ( XTouchEvent ** aToucEvents )
{
  int x           = 0;
  int y           = 0;
  int noOfEvents  = 0;

  /* scale and rotate touch events to match GUI coordinates */
  if ( TouchDown )
  {
    x = TouchEventXPos * GuiWidth  / SurfaceWidth;
    y = TouchEventYPos * GuiHeight / SurfaceHeight;

    if ( TouchState == EW_BSP_TOUCH_IDLE )
      TouchState = EW_BSP_TOUCH_DOWN;
    else
      TouchState = EW_BSP_TOUCH_MOVE;

    TouchEvent[ 0 ].XPos   = x;
    TouchEvent[ 0 ].YPos   = y;
    TouchEvent[ 0 ].State  = TouchState;
    noOfEvents = 1;
  }
  else
  {
    if ( TouchState != EW_BSP_TOUCH_IDLE )
    {
      TouchEvent[ 0 ].State = EW_BSP_TOUCH_UP;
      TouchState = EW_BSP_TOUCH_IDLE;
      noOfEvents = 1;
    }
  }

  /* return the prepared touch events and the number of prepared touch events */
  if ( aToucEvents )
    *aToucEvents = TouchEvent;

  return noOfEvents;
}


/* mli, msy */
